@@ -44,6 +44,8 @@
 #include <sidplay/fformat.h>
 #include <sidplay/myendian.h>
 
+#include <sndfile.h>
+
 #define PROGRAM_VERSION "0.4"
 #define PROGRAM_NAME "msid - " PROGRAM_VERSION
 
@@ -104,6 +106,10 @@ typedef struct msid_machine
   msid_driver *driver;
   char *song;
   uword song_no;
+
+  unsigned int dump;
+  unsigned int secs;
+  
 } msid_machine;
 
 
@@ -155,32 +161,49 @@ void *player_thread(void *arg)
   ubyte* buffer = 0;
   int duration = 0;
 
+  SF_INFO sfinfo;
+  SNDFILE *snd;
+  unsigned int sndfile = 0;
+  char sndfilename[256];
+
+  sidTune *tune;
+
+  memset(&sfinfo, 0, sizeof(SF_INFO));
+
+  // TODO - use mono/stereo support here also
+
+  sfinfo.samplerate = 44100;
+  sfinfo.channels = 2;
+  sfinfo.format = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
+
 #ifdef HILDON
 #ifndef __arm__
   fprintf(stderr, "playback not supported in scratchbox\n");
-    G_LOCK(msid_state);
-    msid_state = READY;
-    G_UNLOCK(msid_state);
-    return NULL;
+  goto exit_player_thread;
 #endif
 #endif
   
   // final sanity check, this should never happen
   if (!m->song) {
-    G_LOCK(msid_state);
-    msid_state = READY;
-    G_UNLOCK(msid_state);
     fprintf(stderr, "critical - no song?!1\n");
-    return NULL;
+    goto exit_player_thread;
   }
 
   // init audio driver
-  init_msid_machine (m);
+  init_msid_machine(m);
 
   buffer = new ubyte[m->driver->bsize()];
+  tune = new sidTune(m->song);
 
-  sidTune tune(m->song);
-  sidEmuInitializeSong(m->engine, tune, m->song_no);
+  if (m->dump && sf_format_check(&sfinfo) == TRUE) {
+    struct sidTuneInfo info;
+    tune->getInfo(info);
+    snprintf(sndfilename, 256, "%s.wav", m->song);
+    snd = sf_open(sndfilename, SFM_WRITE, &sfinfo);
+    sndfile = 1;
+  }
+
+  sidEmuInitializeSong(m->engine, *tune, m->song_no);
   m->engine.resetSecondsThisSong();
 
   while (!ready) {
@@ -189,28 +212,46 @@ void *player_thread(void *arg)
     if (msid_state == STOP) {
       ready = 1;
     }
-
     G_UNLOCK(msid_state);
 
     G_LOCK(msid_pause);
     G_UNLOCK(msid_pause);
 
-    sidEmuFillBuffer(m->engine, tune, buffer, m->driver->bsize());
+    sidEmuFillBuffer(m->engine, *tune, buffer, m->driver->bsize());
+
+    if (!m->dump) {
+
+      // stop playback in case of error
+      if (!m->driver->play_stream(buffer, m->driver->bsize())) {
+	ready = 1;
+      }
+    } else {
+
+      sf_write_raw(snd, buffer, m->driver->bsize());
+
+      // finish dump
+      if (m->engine.getSecondsThisSong() >= m->secs + 1) {
+	ready = 1;
+      }
 
-    // stop playback in case of error
-    if (!m->driver->play_stream(buffer, m->driver->bsize())) {
-      ready = 1;
     }
-    
+
     // TODO - debug byteCount ...
   }
 
+  delete tune;
   delete [] buffer;
 
   m->driver->close();
 
  exit_player_thread :
 
+  if (m->dump) {
+    sf_write_sync(snd);
+    sf_close(snd);
+    m->dump = 0;
+  }
+  
   G_LOCK(msid_state);
   msid_state = READY;
   G_UNLOCK(msid_state);
@@ -265,10 +306,11 @@ update_status_text (const char *text, const char *c)
   gchar buffer[256];
   const char *color;
 
-  if (!c)
+  if (!c) {
     color = C64_FG;
-  else
+  } else {
     color = c;
+  }
 
   if (!strstr(text, "READY")) {
     g_timeout_add (2500, set_ready_text_timeout, NULL);
@@ -424,8 +466,8 @@ load_audio_plugin (GtkWidget *widget,
     return;
 
   G_LOCK(msid_state);
-  if (msid_state != READY)
-  {
+  if (msid_state != READY) {
+
     G_UNLOCK(msid_state);
     update_status_text ("cannot update plugin while playing", "#ffaaaa");
     return;
@@ -434,21 +476,17 @@ load_audio_plugin (GtkWidget *widget,
 
   plugin = gtk_combo_box_get_active_text (GTK_COMBO_BOX(data));
 
-  if (plugin)
-  {
+  if (plugin) {
     driver = (msid_driver*) audio_iface.plugin_object;
     snprintf (buffer, 256, "%s/audio/%s", PLUGIN_FILE_PATH, plugin);
 
-    if (g_file_test (buffer, G_FILE_TEST_EXISTS))
-    {
+    if (g_file_test (buffer, G_FILE_TEST_EXISTS)) {
       close_msid_plugin (&audio_iface);
       init_msid_plugin (&audio_iface, buffer);
 
       snprintf (buffer, 256, "plugin [<b>%s</b>] loaded", plugin);
       update_status_text (buffer, "#aaffaa");
-    }
-    else
-    {
+    } else {
       snprintf (buffer, 256, "error loading [<b>%s</b>]", plugin);
       update_status_text (buffer, "#ffaaaa");
     }
@@ -472,10 +510,8 @@ load_search_plugin (GtkWidget *widget,
 
   plugin = gtk_combo_box_get_active_text (GTK_COMBO_BOX(data));
 
-  if (plugin)
-  {
+  if (plugin) {
     snprintf (buffer, 256, "%s/search/%s", PLUGIN_FILE_PATH, plugin);
-
     driver = (msid_search_plugin *) search_iface.plugin_object;
 
     /*
@@ -483,16 +519,13 @@ load_search_plugin (GtkWidget *widget,
     dlclose (search_iface.dl_handle);
     */
 
-    if (g_file_test (buffer, G_FILE_TEST_EXISTS))
-    {
+    if (g_file_test (buffer, G_FILE_TEST_EXISTS)) {
       close_msid_plugin (&search_iface);
       init_msid_plugin (&search_iface, buffer);
 
       snprintf (buffer, 256, "plugin [<b>%s</b>] loaded", plugin);
       update_status_text (buffer, "#aaffaa");
-    }
-    else
-    {
+    } else {
       snprintf (buffer, 256, "error loading [<b>%s</b>]", plugin);
       update_status_text (buffer, "#ffaaaa");
     }
@@ -511,31 +544,36 @@ pause_song (GtkWidget *widget,
 }
 
 
-/* FIXME */
+// stops playback
 static unsigned int
 stop_sid_machine()
 {
   unsigned int ready = 0, timer = 100, result = 1;
 
   G_LOCK (msid_state);
+
+  // maybe already stopped?
+  if (msid_state != PLAY && msid_state != BUSY) {
+    G_UNLOCK(msid_state);
+    return result;
+  }
+
   msid_state = STOP;
   G_UNLOCK(msid_state);
 
-  while (!ready)
-  {
+  while (!ready) {
     usleep (10000);
 
     G_LOCK(msid_state);
-    if (msid_state == READY)
-    {
+    if (msid_state == READY) {
       ready=1;
     }
     G_UNLOCK(msid_state);
 
-    /* just in case something goes wrong */
+    // just in case something goes wrong - should not happen
     timer--;
-    if (!timer)
-    {
+    if (!timer) {
+      fprintf(stderr, "timer run out, did not stop sid machine properly?\n");
       result = 0;
       break;
     }
@@ -645,22 +683,12 @@ play_selection (GtkTreeView       *tree_view,
 
       model = gtk_tree_view_get_model(tree_view);
 
-      // ATTENTION - selection changes here
-      // this all hacking here is because of HILDON pannablearea sucks
-      // selection possibility from gtktreeview .. this breaks proper native gtk support
-      // now as well :/
-
-      if (current_selection) {
-	gtk_tree_path_free(current_selection);
-      }
-      current_selection = gtk_tree_path_copy(path);
+      select_song (tree_view, path);
 
       // update subsong selector only when selection made,
       // not on cursor change
       modify_subsong_selector();
 
-      select_song (tree_view, NULL);
-
       if (gtk_tree_model_get_iter (model, &iter, path)) {
 
 	gtk_tree_model_get(model, &iter, PLAYER_LIST_SONG_PATH, &filepath, -1);
@@ -811,8 +839,6 @@ modify_subsong_selector()
       prev_amount--;
     }
 
-    // TODO - set same subsong if possible!
-
     // ignore this change as it's program making it
     ignore_subsong_change = TRUE;
 
@@ -830,7 +856,6 @@ delete_current_selection (void)
 {
   GtkTreeSelection *selection;
   GtkTreeModel     *model;
-  GtkTreePath      *path;
   GtkTreeIter       iter, next;
   gchar *filepath=NULL;
   gchar *name=NULL;
@@ -933,10 +958,46 @@ key_on_selection(GtkWidget   *widget,
 }
 
 
+/*
+ * This function may seem horrible but I am trying to
+ * get rid of dependency to gtktreeview selection since
+ * hildon pannablearea does not support selection and I
+ * hope to use pannablearea soon for lists.
+ */
 static void
 select_song (GtkTreeView *treeview,
              gpointer data)
 {
+  GtkTreePath *path = (GtkTreePath*) (data);
+
+  if (path == NULL) {
+    // get cursor selection from treeview if possible
+    GtkTreeSelection *selection;
+    GtkTreeModel     *model;
+    GtkTreeIter       iter;
+    GtkTreePath *spath;
+    GtkTreeViewColumn *column;
+    gchar *filepath=NULL;
+    GError *error = NULL;
+
+    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(player_song_list));
+    if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
+      spath = gtk_tree_model_get_path(model, &iter);
+      if(!spath) {
+	printf("no path :( \n");
+      }
+      current_selection = gtk_tree_path_copy(spath);
+      gtk_tree_path_free(spath);
+      goto have_selection;
+    }
+  }
+
+  if (current_selection) {
+    gtk_tree_path_free(current_selection);
+  }
+  current_selection = gtk_tree_path_copy(path);
+
+  have_selection :
 
   /* since cursor changed - we can release image_button for the user again */
   gtk_widget_set_sensitive (image_button, TRUE);
@@ -1076,6 +1137,118 @@ show_info_dialog (const char *title, const char *txt1, const char *txt2)
   gtk_widget_destroy(dialog);
 }
 
+
+
+void calculate_size (GtkRange *range, gpointer data)
+{
+  GtkWidget *label = GTK_WIDGET(data);
+  char buffer[256];
+
+  snprintf(buffer, 256, "~%2.1fMB", (gtk_range_get_value(range) * 204.8)/1024);
+  gtk_label_set_text(GTK_LABEL(label), buffer);
+
+}
+
+void
+create_ringtone (GtkMenuItem *menuitem, gpointer data)
+{
+  GtkTreeSelection *selection;
+  GtkTreeModel     *model;
+  GtkTreeIter       iter, next;
+  GError *error = NULL;
+
+  msid_machine *m = (msid_machine*) data;
+
+  gchar *filepath=NULL;
+  gchar *name=NULL;
+  char buffer[256];
+
+  GtkWidget *dialog;
+  GtkWidget *label;
+  GtkWidget *sizelabel;
+  GtkWidget *scale = gtk_hscale_new_with_range(1.0, 120.0, 1.0);
+
+  gtk_range_set_value(GTK_RANGE(scale), m->secs);
+
+  // TODO - show progress indication during dump thread
+  // TODO - set as current ringtone automatically if possible?
+
+  // cannot playback while dumping
+  stop_sid_machine();
+
+  if (!current_selection) {
+    show_info_dialog("No song selected", "You need to first select a song from the list.", "");
+    return;
+  }
+
+  model = gtk_tree_view_get_model (GTK_TREE_VIEW(player_song_list));
+
+  if (gtk_tree_model_get_iter (model, &iter, current_selection)) {
+    gtk_tree_model_get(model, &iter, PLAYER_LIST_SONG_PATH, &filepath, -1);
+    gtk_tree_model_get(model, &iter, PLAYER_LIST_SONG_NAME, &name, -1);
+  }
+
+  if (!filepath || !name) {
+    return;
+  }
+
+  if (m->song) {
+    g_free(m->song);
+  }
+
+  m->song = filepath;
+  m->song_no = SUBSONG;
+
+  snprintf(buffer, 256, "Msid will create\n%s.wav\nLength in secs :", name);
+  label = gtk_label_new("");
+
+  gtk_label_set_markup (GTK_LABEL(label), buffer);
+  gtk_label_set_line_wrap (GTK_LABEL(label), TRUE);
+  
+  sizelabel = gtk_label_new("");
+
+  calculate_size(GTK_RANGE(scale), sizelabel);
+  
+  g_signal_connect(G_OBJECT (scale), "value-changed",
+		   G_CALLBACK (calculate_size), sizelabel);
+
+  dialog = gtk_dialog_new_with_buttons ("Create ringtone",
+					GTK_WINDOW(app_window),
+					(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
+					GTK_STOCK_OK,
+					GTK_RESPONSE_ACCEPT,
+					GTK_STOCK_CANCEL,
+					GTK_RESPONSE_REJECT,
+					NULL);
+  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), label);
+  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), scale);
+  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), sizelabel);
+
+  gtk_widget_show_all (dialog);
+
+  gint result = gtk_dialog_run (GTK_DIALOG (dialog));
+
+  switch (result)
+    {
+    case GTK_RESPONSE_ACCEPT:
+    case GTK_RESPONSE_OK:
+
+      m->dump = 1;
+      m->secs = gtk_range_get_value((GtkRange*)scale);
+
+      play_stop_song (NULL, data);
+
+      break;
+    default :
+      break;
+    }
+
+    gtk_widget_destroy (dialog);
+
+    update_status_text("RINGTONE CREATED", C64_FG);
+}
+
+
 void
 show_about_dialog (GtkMenuItem *menuitem, gpointer data)
 {
@@ -1122,7 +1295,7 @@ show_help_dialog (GtkMenuItem *menuitem, gpointer data)
   GtkWidget *info;
   GtkWidget *hbox;
 
-  char txt[] = "Use <span foreground=\"#aaaaff\">Search</span> tab to download sid files, when tapped they will appear on <span foreground=\"#aaaaff\">Player</span> tab. When a song is selected in player tab, use <span foreground=\"#aaaaff\">Screenshot</span> tab to display screenshots related to the song, works with most game sid tunes.\n\nEnjoy!";
+  char txt[] = "Use <span foreground=\"#aaaaff\">Search</span> tab to download sid files, when tapped they will appear on <span foreground=\"#aaaaff\">Player</span> tab. When a song is selected in player tab, use <span foreground=\"#aaaaff\">Screenshot</span> tab to display screenshots related to the song, works with most game sid tunes.\n\n<span foreground=\"#aaaaff\">Create ringtone</span> feature creates wav file to same directory as sidtune was found from.\n\nEnjoy!";
 
   GtkWidget *dialog = gtk_dialog_new_with_buttons ("Help!",
 						   GTK_WINDOW(app_window),
@@ -1268,6 +1441,7 @@ int main (int argc, char **argv)
     *item2,
     *item3,
     *item4,
+    *item5,
 
     *player_main_box,
     *status_background,
@@ -1304,6 +1478,8 @@ int main (int argc, char **argv)
   msid_state = READY;
 
   machine.song = 0;
+  machine.dump = 0;
+  machine.secs = 5; // default
 
   g_thread_init(NULL);
   gdk_threads_init();
@@ -1515,9 +1691,10 @@ int main (int argc, char **argv)
   item1 = gtk_menu_item_new_with_label ("Help");
   item2 = gtk_menu_item_new_with_label ("About");
   item3 = gtk_menu_item_new_with_label ("Preferences");
+  item4 = gtk_menu_item_new_with_label ("Create Ringtone");
 
 #ifndef __arm__
-  item4 = gtk_menu_item_new_with_label ("Quit");
+  item5 = gtk_menu_item_new_with_label ("Quit");
 #endif
 
 
@@ -1527,9 +1704,11 @@ int main (int argc, char **argv)
                     G_CALLBACK (show_about_dialog), NULL);
   g_signal_connect (G_OBJECT (item3), "activate",
                     G_CALLBACK (show_config_dialog), config_page);
+  g_signal_connect (G_OBJECT (item4), "activate",
+                    G_CALLBACK (create_ringtone), &machine);
 
 #ifndef __arm__
-  g_signal_connect (G_OBJECT (item4), "activate",
+  g_signal_connect (G_OBJECT (item5), "activate",
                     G_CALLBACK (quit_msid), NULL);
 #endif
 
@@ -1539,6 +1718,7 @@ int main (int argc, char **argv)
   gtk_menu_append (GTK_MENU (menu_bar), item1);
   gtk_menu_append (GTK_MENU (menu_bar), item2);
   gtk_menu_append (GTK_MENU (menu_bar), item3);
+  gtk_menu_append (GTK_MENU (menu_bar), item4);
 
   hildon_program_set_common_menu (program, GTK_MENU (menu_bar));
 #else
@@ -1550,6 +1730,7 @@ int main (int argc, char **argv)
   gtk_menu_shell_append (GTK_MENU_SHELL(menu), item2);
   gtk_menu_shell_append (GTK_MENU_SHELL(menu), item3);
   gtk_menu_shell_append (GTK_MENU_SHELL(menu), item4);
+  gtk_menu_shell_append (GTK_MENU_SHELL(menu), item5);
 
   gtk_menu_item_set_submenu (GTK_MENU_ITEM(main_menu), menu);
   gtk_menu_shell_append (GTK_MENU_SHELL(menu_bar), main_menu);
@@ -1575,10 +1756,8 @@ int main (int argc, char **argv)
   g_signal_connect (G_OBJECT (player_song_list), "row-activated",
                     G_CALLBACK (handle_selection), (gpointer) &machine);
 
-  /*
-    g_signal_connect (G_OBJECT (player_song_list), "cursor-changed",
-    G_CALLBACK (select_song), NULL);
-  */
+  g_signal_connect (G_OBJECT (player_song_list), "cursor-changed",
+		    G_CALLBACK (select_song), NULL);
 
   g_signal_connect (G_OBJECT (player_play_button), "clicked",
                     G_CALLBACK (play_stop_song), (gpointer) &machine);
@@ -1635,6 +1814,8 @@ int main (int argc, char **argv)
   driver  = (msid_driver *) audio_iface.plugin_object;
   splugin = (msid_search_plugin *) search_iface.plugin_object;
 
+
+
   gtk_main();
   gdk_threads_leave();
 
